/*****************************************************************************
 * freecnc.cpp - Initialize SDL, initial startup of game, clean up at end
 *
 ****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include "SDL.h"
#include "freecnc.h"

#include "game.h"

#include "vqa.h"
#include "wsa.h"
#include "vfs.h"
#include "memtrack.h"


/* Main function, creates a instance of the FreeCNC class and runs it.
 * After parseing the commandline args. */
int main( int argc, char **argv ) {
    FreeCNC *freecnc;
    Args *arg = new Args();

    fprintf(stderr,"Please wait, FreeCNC is starting\n");
    fprintf(stderr,"Initialising the VFS...");
    VFS_Init();
    fprintf(stderr,"done\n");
    /* -1 is an errorcode, abort */
    if( arg->parse(argc, argv) == -1 ) {
        delete arg;
        exit(0);
    }
    /* Create a new freecnc session, run it and remove it from memory */
    freecnc = new FreeCNC(arg);
    freecnc->run();
    delete freecnc;
    VFS_Destroy();
#if defined _DEBUG && !defined _WIN32

    DumpUnfreed();
#endif

    exit(0);
}

/** Constructor, initiates sdl, sets up a timer and gets ready to run.
 * Also plays the intro movie
 * @param a class containing arguments.
 */
FreeCNC::FreeCNC(Args *arg) {
    VQAMovie *mov;
    WSAImage *choose;

    this->arg = arg;

    /* Initialize SDL */
    if( SDL_Init(SDL_INIT_VIDEO|SDL_INIT_AUDIO|SDL_INIT_TIMER) < 0 ) {
        fprintf( stderr, "Couldn't initialize SDL: %s\n", SDL_GetError() );
        delete arg;
        exit(1);
    }
    /* Hide the cursor since we have out own */
    SDL_ShowCursor( 0 );
    /* Init the video and sound */
    try {
        fprintf(stderr,"Initialising the graphics engine...");
        gfxengine = new GraphicsEngine(arg);
        fprintf(stderr,"done\n");
    } catch( VideoError *e ) {
        fprintf(stderr,"failed.  exiting\n");
        delete e;
        delete arg;
        SDL_Quit();
        exit(1);
    }
    try {
        fprintf(stderr,"Initialising the sound engine...");
        sndengine = new SoundEngine(arg->getNosound());
        if (!arg->getNosound()) {
            fprintf(stderr,"done\n");
        }
    } catch( SoundError *e ) {
        fprintf(stderr,"failed. exiting");
        delete e;
        delete arg;
        SDL_Quit();
        exit(1);
    }
    if (arg->getVQAPlayer() ) {
        fprintf(stderr,"now playing %s\n",arg->getVQAMovie());
        try {
            mov = new VQAMovie(gfxengine, sndengine, arg->getVQAMovie());
            mov->play();
            delete mov;
        } catch (VQAError* vqaerr) {
            delete vqaerr;
            exit(1);
        }
        delete arg;
        delete gfxengine;
        delete sndengine;
        SDL_Quit();
        exit(0);
    }
    /* Play the intro vqa if we want to do so */
    if( arg->getIntro() ) {
        try {
            if (arg->getGameNum() == GAME_RA)
                mov = new VQAMovie(gfxengine, sndengine, "prolog");
            else
                mov = new VQAMovie(gfxengine, sndengine, "logo");
            mov->play();
            delete mov;
        } catch (VQAError *vqaerr) {
            delete vqaerr;
        }

        try {
            choose = new WSAImage(gfxengine, sndengine, "choose.wsa");
            choose->Animate();
            delete choose;
        } catch( WSAError *wsaerr ) {
            delete wsaerr;
        }
    }

    /* Init the rand functions */
    srand(time(NULL));
}

/** Destructor, quits sdl and frees up some memory */
FreeCNC::~FreeCNC() {
    delete arg;
    delete gfxengine;
    delete sndengine;
    SDL_Quit();
}

/** Run a freecnc session */
void FreeCNC::run() {
    fprintf(stderr,"Initialising game engine:\n");
    Game *gsession = new Game(gfxengine, sndengine, arg->getMapName(), arg->getGameNum());
    fprintf(stderr,"Starting game engine\n");
    gsession->play();
    fprintf(stderr,"Shutting down\n");
    delete gsession;
}
