#include "unitandstructurepool.h"

#include "weaponspool.h"
#include "game.h"
#include "memtrack.h"

using namespace std;

/** Constructor, loads all the units from the inifile and create
 * those units/structures in the unit/structure pool 
 * @param ccmap pointer to the map object 
 * @param evQueue pointer to the event queue */
UnitAndStructurePool::UnitAndStructurePool(
    CnCMap *ccmap,
    ActionEventQueue *evQueue) {

    this->ccmap = ccmap;

    //   this->imagepool = ccmap->getImagePool();

    unitandstructmat = new Uint32[ccmap->getWidth() * ccmap->getHeight()];
    memset(unitandstructmat, 0, sizeof(Uint32)*ccmap->getWidth()*ccmap->getHeight());

    try {
        structini = new INIFile("structure.ini");
        unitini = new INIFile("unit.ini");
    } catch( INInotFound *e ) {
        delete e;
        exit(1);
    }
    strcpy(theaterext, ".");
    strncat(theaterext, ccmap->getTheater(), 3);
    Unit::setUASP(this); // static member function
    Structure::setUASP(this); // static member function
    this->evQueue = evQueue;

    deleted_unitorstruct = false;
    numdeletedunit = 0;
    numdeletedstruct = 0;
    wp = new WeaponsPool(ccmap->getImagePool());
}


/** Destructor, empties the various matricies and deletes the WeaponsPool
 * and inifile members.
 */
UnitAndStructurePool::~UnitAndStructurePool() {
    Uint32 i;
    delete[] unitandstructmat;
    delete structini;
    delete unitini;
    for( i = 0; i < unitpool.size(); i++ ) {
        if(unitpool[i] != NULL ) {
            delete unitpool[i];
        }
    }
    for( i = 0; i < unittypepool.size(); i++ )
        delete unittypepool[i];
    for( i = 0; i < structurepool.size(); i++ ) {
        if(structurepool[i] != NULL) {
            delete structurepool[i];
        }
    }
    for( i = 0; i < structuretypepool.size(); i++ )
        delete structuretypepool[i];
    delete wp;
}



/** @brief Retrieve the imagenumber for the structure at the specified position
 * @param cellpos the position we want to know the structure of.
 * @param inumbers pointer to array of image numbers 
 * @param xoffsets pointer to array of x offsets
 * @param yoffsets pointer to array of y offsets
 * @returns the imagenumber.  */
Uint8 UnitAndStructurePool::getUnitOrStructureNum(Uint16 cellpos, Uint32 **inumbers,
        Sint8 **xoffsets, Sint8 **yoffsets) {
    Structure *st;
    Unit *un;

    Uint8 layers, i;

    if( !(unitandstructmat[cellpos]&US_LOWER_RIGHT) )
        return 0;
    if( unitandstructmat[cellpos]&US_IS_UNIT ) {
        un = unitpool[(unitandstructmat[cellpos])&0xffff];
        if( ((UnitType *)un->getType())->isInfantry() )
            return un->getInfantryGroup()->getImageNums(inumbers, xoffsets, yoffsets);
        return un->getImageNums(inumbers, xoffsets, yoffsets);
    } else if( unitandstructmat[cellpos]&(US_IS_STRUCTURE|US_IS_WALL) ) {
        st = structurepool[(unitandstructmat[cellpos])&0xffff];
        layers = st->getImageNums(inumbers);
        (*xoffsets) = new Sint8[layers];
        (*yoffsets) = new Sint8[layers];
        for( i = 0; i < layers; i++ ) {
            (*xoffsets)[i] = ((StructureType *)st->getType())->getXoffset();
            (*yoffsets)[i]  = ((StructureType *)st->getType())->getYoffset();
        }
        return layers;
    }
    return 0;
}

/// Gets a list of all flying stuff in the current tile
Uint8 UnitAndStructurePool::getL2overlays(Uint16 pos, Uint32 **inumbers, Sint8 **xoffset, Sint8 **yoffset) {
    multimap<Uint16, L2Overlay*>::iterator entry;
    //multimap<Uint16, L2Overlay*>::iterator lastentry;
    L2Overlay *curl2;
    Uint8 numentries, i;
    if( !(unitandstructmat[pos]&US_HAS_L2OVERLAY) ) {
        return 0;
    }
    entry = l2pool.find(pos);
    numentries = l2pool.count(pos);
    *inumbers = new Uint32[numentries];
    *xoffset = new Sint8[numentries];
    *yoffset = new Sint8[numentries];
    for( i = 0; i < numentries; i++ ) {
        curl2 = entry->second;
        (*inumbers)[i] = curl2->imagenum;
        (*xoffset)[i] = curl2->xoffset;
        (*yoffset)[i] = curl2->yoffset;
        entry++;
    }
    return numentries;
}

multimap<Uint16, L2Overlay*>::iterator UnitAndStructurePool::addL2overlay(Uint16 cellpos, L2Overlay *ov) {
    multimap<Uint16, L2Overlay*>::iterator entry;
    l2pool.insert(make_pair(cellpos, ov));
    unitandstructmat[cellpos]|=US_HAS_L2OVERLAY;
    entry = l2pool.find(cellpos);
    while(entry->second != ov) {
        entry++;
    }
    return entry;
}

void UnitAndStructurePool::removeL2overlay(std::multimap<Uint16, L2Overlay*>::iterator entry) {
    Uint32 cellpos = entry->first;
    l2pool.erase(entry);
    if( l2pool.find(cellpos) == l2pool.end() ) {
        unitandstructmat[cellpos]&=(~US_HAS_L2OVERLAY);
    }
}

/// wrapper function that assumes that the build animation is not to be played
void UnitAndStructurePool::createStructure( char *typen, Uint16 cellpos, Uint8 owner ) {
    createStructure( typen, cellpos, owner, false);
}

/// Creates a structure
void UnitAndStructurePool::createStructure( char *typen, Uint16 cellpos, Uint8 owner, bool makeanim) {
    int x, y, curpos;
    bool setlr;
    Uint16 structnum;

    StructureType *type;
    Structure *st;

    if( numdeletedstruct > 0 ) {
        for( structnum = 0; structnum < structurepool.size(); structnum++ ) {
            if(structurepool[structnum] == NULL)
                break;
        }
    } else {
        structnum = structurepool.size();
    }

    type = getStructureTypeByName(typen);
    st = new Structure(type, cellpos, owner);
    //   unitandstructmat[cellpos] = structurepool.size();
    if( type->isWall() )
        unitandstructmat[cellpos] = (US_LOWER_RIGHT|US_IS_WALL)|structnum;
    else {
        //      unitandstructmat[cellpos] |= US_UPPER_LEFT;
        curpos = cellpos+ccmap->getWidth()*type->getYsize();
        setlr = false;
        for( y = type->getYsize()-1; y>=0; y-- ) {
            curpos -= ccmap->getWidth();
            for(x = type->getXsize()-1; x>=0; x--) {
                if( type->isBlocked(y*type->getXsize()+x) ) {
                    unitandstructmat[curpos+x] = US_IS_STRUCTURE|structnum;
                    if( !setlr ) {
                        unitandstructmat[curpos+x] |= US_LOWER_RIGHT;
                        setlr=true;
                    }
                }
            }
        }
        //	  unitandstructmat[curpos-mapwidth] |= US_LOWER_RIGHT;
    }

    st->setStructnum(structnum);
    if( structnum == structurepool.size() ) {
        structurepool.push_back(st);
    } else {
        structurepool[structnum] = st;
        numdeletedstruct--;
    }

    // update the wall-images
    if( type->isWall() ) {
        updateWalls(st,true);
    } else {
        if (makeanim) {
            st->runAnim(0);
        } else {
            if( (type->getAnimInfo().animtype == 1) || (type->getAnimInfo().animtype == 4) ) {
                st->runAnim(1);
            }
        }
    }
}


/// Creates a unit
void UnitAndStructurePool::createUnit( char *typen, Uint16 cellpos, Uint8 subpos, Uint8 owner ) {
    Uint16 unitnum;

    UnitType *type;
    Unit *un;

    //   int x, y, curpos;
    if( numdeletedunit > 0 ) {
        for( unitnum = 0; unitnum < unitpool.size(); unitnum++ ) {
            if(unitpool[unitnum] == NULL)
                break;
        }
    } else {
        unitnum = unitpool.size();
    }

    InfantryGroup *group;

    type = getUnitTypeByName(typen);

    group = NULL;
    if( type->isInfantry() ) {
        if(unitandstructmat[cellpos]&US_IS_UNIT)
            group = unitpool[unitandstructmat[cellpos]&0xffff]->getInfantryGroup();
        else
            group = new InfantryGroup();
    }

    un = new Unit(type, cellpos, subpos, group, owner);
    unitandstructmat[cellpos] = unitnum;
    unitandstructmat[cellpos] |= US_LOWER_RIGHT|US_IS_UNIT;
    /*	curpos = cellpos;
    	for( y = 0; y < type->getSize(); y++ ){
    		for(x = 0; x < type->getSize(); x++){
    			unitandstructmat[curpos+x] |= US_IS_UNIT;
    		}
    		curpos += mapwidth;
    	}*/
    un->setUnitnum(unitnum);
    if( unitnum == unitpool.size() ) {
        unitpool.push_back(un);
    } else {
        unitpool[unitnum] = un;
        numdeletedunit--;
    }
}

Unit *UnitAndStructurePool::getUnitAt(Uint32 cell, Uint8 subcell) {
    Unit *un;
    if( !(unitandstructmat[cell] & US_IS_UNIT) )
        return NULL;
    un = unitpool[unitandstructmat[cell]&0xffff];
    if( ((UnitType *)un->getType())->isInfantry() )
        return un->getInfantryGroup()->unitAt(subcell);
    return un;

}

/** @brief wrapper function that assumes that walls are not wanted
 * @param cell the cell to be examined for structures 
 * @returns a pointer to the Unit if found */
Structure* UnitAndStructurePool::getStructureAt(Uint32 cell) {
    return getStructureAt(cell,false);
}

/** @brief retrieves the structure at a given cell (NULL if there is no structure).
 * @param cell the cell to be examined for structures
 * @param wall if false, will return NULL if a wall is found in the cell 
 * @returns a pointer to the Structure if found */
Structure *UnitAndStructurePool::getStructureAt(Uint32 cell, bool wall) {
    if( !(unitandstructmat[cell] & US_IS_STRUCTURE) &&
            !(wall && (unitandstructmat[cell]&US_IS_WALL)))
        return NULL;
    return structurepool[unitandstructmat[cell]&0xffff];

}

/// retrieves units or structures from a given cell
/** like getUnitAt and getStructureAt except works for either.
 * the big difference is that the return type is UnitOrStructure* (with which fewer
 * things can be done).
 * @param cell the cell to be examined
 * @param subcell the subposition of the cell to be examined (only valid for infantry)
 */
UnitOrStructure* UnitAndStructurePool::getUnitOrStructureAt(Uint32 cell, Uint8 subcell) {
    Unit* un;

    Uint32 cval = unitandstructmat[cell];
    if (cval & US_IS_UNIT) {
        un = unitpool[unitandstructmat[cell]&0xffff];
        if( ((UnitType *)un->getType())->isInfantry() )
            return (UnitOrStructure*)un->getInfantryGroup()->unitAt(subcell);
        return (UnitOrStructure*)un;
    } else if (cval & US_IS_STRUCTURE) {
        return (UnitOrStructure*)structurepool[unitandstructmat[cell]&0xffff];
    } else {
        return NULL;
    }
}

Uint16 UnitAndStructurePool::getSelected(Uint32 pos) {
    int i;
    Uint16 selected;

    if( !(unitandstructmat[pos]&US_LOWER_RIGHT) )
        return 0;
    if( unitandstructmat[pos]&US_IS_STRUCTURE ) {
        if(structurepool[unitandstructmat[pos]&0xffff]->isSelected())
            return (((StructureType *)structurepool[unitandstructmat[pos]&0xffff]->getType())->getXsize()<<8) | 1;
    } else if( unitandstructmat[pos]&US_IS_UNIT ) {
        if(((UnitType *)unitpool[unitandstructmat[pos]&0xffff]->getType())->isInfantry()) {
            selected = 0xff00;
            for( i = 0; i < 5; i++ )
                if( !unitpool[unitandstructmat[pos]&0xffff]->getInfantryGroup()->isClear(i) )
                    if(unitpool[unitandstructmat[pos]&0xffff]->getInfantryGroup()->unitAt(i)->isSelected() )
                        selected |= 1<<i;

            return selected;
        } else if(unitpool[unitandstructmat[pos]&0xffff]->isSelected())
            return (1<<8) | 1;
    }
    return 0;
}

/** called by MoveAnimEvent before moving to set up the unitandstructure matrix
 * @param un the unit about to move
 * @param dir the direction in which the unit is to move
 * @param xmod the modifier used to adjust the xoffset (set in this function)
 * @param ymod the modifier used to adjust the yoffset (set in this function)
 * @returns the cell of the new position */
Uint16 UnitAndStructurePool::preMove(Unit *un, Uint8 dir, Sint8 *xmod, Sint8 *ymod) {
    Uint16 newpos;
    Sint8 unitmod = ((UnitType *)un->getType())->getMoveMod();

    switch(dir) {
    case MOVE_UP:
        newpos = un->getPos()-ccmap->getWidth();
        *xmod = 0;
        *ymod = -unitmod;
        break;
    case MOVE_UPRIGHT:
        newpos = un->getPos()-ccmap->getWidth()+1;
        *xmod = unitmod;
        *ymod = -unitmod;
        break;
    case MOVE_RIGHT:
        newpos = un->getPos()+1;
        *xmod = unitmod;
        *ymod = 0;
        break;
    case MOVE_DOWNRIGHT:
        newpos = un->getPos()+ccmap->getWidth()+1;
        *xmod = unitmod;
        *ymod = unitmod;
        break;
    case MOVE_DOWN:
        newpos = un->getPos()+ccmap->getWidth();
        *xmod = 0;
        *ymod = unitmod;
        break;
    case MOVE_DOWNLEFT:
        newpos = un->getPos()+ccmap->getWidth()-1;
        *xmod = -unitmod;
        *ymod = unitmod;
        break;
    case MOVE_LEFT:
        newpos = un->getPos()-1;
        *xmod = -unitmod;
        *ymod = 0;
        break;
    case MOVE_UPLEFT:
        newpos = un->getPos()-ccmap->getWidth()-1;
        *xmod = -unitmod;
        *ymod = -unitmod;
        break;
    default:
        return 0xffff;
        break;
    }
    /* if an infantry's position got updated */
    if( unitandstructmat[newpos]&(US_IS_WALL|US_IS_STRUCTURE|US_IS_UNIT|US_MOVING_HERE) ) {
        if( !((UnitType *)un->getType())->isInfantry() ) {
            // remove this later when code for moving over walls and infantry is done
            return 0xffff;
        }
        if (unitandstructmat[newpos]&(US_IS_WALL)) {
            /** @todo check for tracked and wall type to allow over running some walls
             */
            return 0xffff;
        }
        if (unitandstructmat[newpos]&(US_IS_STRUCTURE)) {
            return 0xffff;
        }
        if( unitandstructmat[newpos]&US_IS_UNIT ) {
            if( !((UnitType *)unitpool[unitandstructmat[newpos]&0xffff]->getType())->isInfantry() ) {
                /** @todo infantry squishing, check moving unit's type for tracked and accept
                 *  this cell as valid if so.
                 **/
                return 0xffff;
            }
            if ( ((Unit* )unitpool[unitandstructmat[newpos]&0xffff])->getOwner() != un->getOwner()) {
                // do not allow units of different sides to occupy same cell
                // this is not allowed because apart from looking weird,
                // area of effect weapons (e.g. flame thrower) will take
                // themselves with their target(s).
                return 0xffff;
            }
            if( (((unitandstructmat[newpos]&US_MOVING_HERE)>>24) +
                    unitpool[unitandstructmat[newpos]&0xffff]->getInfantryGroup()->getNumInfantry()) >= 5)
                return 0xffff;

        } else if(((unitandstructmat[newpos]&US_MOVING_HERE)>>24) >= 5)
            // more than five infantry in current cell
            return 0xffff;

    }

    if( ((UnitType *)un->getType())->isInfantry() )
        unitandstructmat[newpos] += 0x1000000;
    else
        unitandstructmat[newpos] |= US_MOVING_HERE;

    return newpos;
}

/** Called when a unit has moved into a new cell
 * @param un the unit
 * @param newpos the cell into which the unit has moved 
 * @returns the new sub position for the unit (only needed for infantry) */
Uint8 UnitAndStructurePool::postMove(Unit *un, Uint16 newpos) {
    Uint8 subpos = 0;
    Uint8 i;
    InfantryGroup *ig;
    if( ((UnitType *)un->getType())->isInfantry() ) {
        subpos = un->getSubpos();
        ig = un->getInfantryGroup();
        if( ig->getNumInfantry() == 1 ) {
            // old cell is now empty
            unitandstructmat[un->getPos()] &= ~(US_IS_UNIT|US_LOWER_RIGHT);
            delete un->getInfantryGroup();
        } else {
            ig->removeInfantry(subpos);
            if( un->getUnitnum() == (unitandstructmat[un->getPos()]&0xffff) ) {
                unitandstructmat[un->getPos()]&=0xffff0000;
                for( i = 0; i < 5; i++ )
                    if( !ig->isClear(i) )
                        break;
                unitandstructmat[un->getPos()]|=ig->unitAt(i)->getUnitnum();
            }
        }
        /* check indirectly for infantry group
         if the new cell has the US_IS_UNIT flag set, it is assumed
         that there is an infantry group for that cell */
        if( unitandstructmat[newpos] & US_IS_UNIT ) {
            ig = unitpool[unitandstructmat[newpos]&0xffff]->getInfantryGroup();
            // search for an empty sub position for the unit
            for( subpos = 0; subpos < 5; subpos++ )
                if( ig->isClear(subpos) )
                    break;
        } else { // infantry group does not exist for this cell yet
            ig = new InfantryGroup();
            unitandstructmat[newpos] &= 0xffff0000;
            unitandstructmat[newpos] |= un->getUnitnum();
            // note that the subpos variable stays zero (from start of function)
        }
        ig->addInfantry(un, subpos);
        un->setInfantryGroup(ig);
        unitandstructmat[newpos] |= US_LOWER_RIGHT|US_IS_UNIT;

        unitandstructmat[newpos] -= 0x1000000;
    } else {
        // clear values from old position
        unitandstructmat[un->getPos()] &= ~(US_LOWER_RIGHT|US_IS_UNIT);
        /* easier to assign directly than bitwise AND the compliment of US_MOVING_HERE
         then bitwise OR this value. */
        unitandstructmat[newpos] = US_LOWER_RIGHT|US_IS_UNIT|un->getUnitnum();
    }

    return subpos;
}

/** resets the US_MOVING_HERE flag of a cell when the unit stops
 *  before it reaches its destination.
 *  @param un the unit stopping.
 *  @param pos the position to which the unit was moving before stopping.
 *  @bug Abort moving large groups of units causes strange behaviour.
 **/

void UnitAndStructurePool::abortMove(Unit* un, Uint32 pos) {
    //fprintf(stderr,"Aborting move by %p to %u\n",un,pos);
    if (((UnitType *)un->getType())->isInfantry()) {
        //fprintf(stderr,"abMi x == %x\n",((unitandstructmat[pos] & 0x7000000) >> 24));
    }
    else {
        if (!unitandstructmat[pos] & US_MOVING_HERE) {
            //fprintf(stderr,"Doing nothing\n");
        }
        else {
            unitandstructmat[pos] &= ~(US_MOVING_HERE);
            //fprintf(stderr,"reset: %x\n",unitandstructmat[pos]);
        }
    }
}

Uint16 UnitAndStructurePool::getTileCost(Uint16 pos) {
    return getTileCost(pos,NULL);
}

Uint16 UnitAndStructurePool::getTileCost(Uint16 pos, Unit* excpUn) {
    Unit *un;
    if( unitandstructmat[pos] & (US_IS_WALL | US_IS_STRUCTURE) )
        return 0xfff0;
    if( unitandstructmat[pos] & US_MOVING_HERE ) {
        return 2;
    }
    if( unitandstructmat[pos] & US_IS_UNIT ) {
        un = unitpool[unitandstructmat[pos]&0xffff];
        if (un == excpUn)
            return 0;
        if( un->getOwner() == costcalcowner )
            return 2;
        return 10;
    }
    return 0;
}

/*
Uint16 *UnitAndStructurePool::getCostMatrix(Uint8 unittype)
{
   Uint16 lastpos = ccmap->getWidth()*ccmap->getHeight();
   Uint16 i;
   Uint16 *cost = new Uint16[lastpos];
   
   for( i = 0; i < lastpos; i++ ){
      if( unitandstructmat[i] & (US_IS_WALL|US_IS_UNIT|US_IS_STRUCTURE) )
	cost[i] = 0xffff;
      else
	cost[i] = 1;
	//cost = ccmap->getCost(i);
   }
   return cost;
}
*/

/** @brief searches the UnitType pool for a unit type with a given name.
 *  if the type can not be found, it is read in from units.ini
 * @param unitname the name of the unit to retrieve 
 * @returns pointer to the UnitType value */
UnitType* UnitAndStructurePool::getUnitTypeByName(char* unitname) {
    map<string, Uint16>::iterator typeentry;
    UnitType* type;
    Uint16 typenum;

    typeentry = unitname2typenum.find((string)unitname);

    if( typeentry != unitname2typenum.end() ) {
        typenum = typeentry->second;
        type = unittypepool[typenum];
    } else {
        typenum = unittypepool.size();
        type = new UnitType(ccmap->getImagePool(), unitname, unitini, wp, this);
        unittypepool.push_back(type);
        unitname2typenum[(string)unitname] = typenum;
    }
    return type;
}

/** @brief same as getUnitTypeByName but for structures (and the ini file is structure.ini)
 * @param structname the name of the structure to retrieve (e.g. FACT or PROC)
 * @returns pointer to the StructureType value */
StructureType *UnitAndStructurePool::getStructureTypeByName(char* structname) {
    Uint16 typenum;
    std::map<std::string, Uint16>::iterator typeentry;

    StructureType *type;

    typeentry = structname2typenum.find((string)structname);

    if( typeentry != structname2typenum.end() ) {
        typenum = typeentry->second;
        type = structuretypepool[typenum];
    } else {
        typenum = structuretypepool.size();
        type = new StructureType(ccmap->getImagePool(), structname, structini, theaterext, wp);
        structuretypepool.push_back(type);
        structname2typenum[(string)structname] = typenum;
    }
    return type;
}

/// removes a unit from the map
void UnitAndStructurePool::removeUnit(Unit *un) {
    int i;
    InfantryGroup *infgrp;
    unitpool[un->getUnitnum()] = NULL;
    if( ((UnitType *)un->getType())->isInfantry() ) {
        infgrp = un->getInfantryGroup();
        infgrp->removeInfantry(un->getSubpos());
        if( infgrp->getNumInfantry() == 0 ) {
            delete infgrp;
            unitandstructmat[un->getPos()] &= ~(US_LOWER_RIGHT|US_IS_UNIT);
        } else if( (unitandstructmat[un->getPos()]&0xffff) == un->getUnitnum() ) {
            for( i = 0; i < 5; i++ ) {
                if(!infgrp->isClear(i)) {
                    unitandstructmat[un->getPos()]&=~0xffff;
                    unitandstructmat[un->getPos()]|=infgrp->unitAt(i)->getUnitnum();
                    break;
                }
            }
        }
        un->setInfantryGroup(NULL);
    } else {
        unitandstructmat[un->getPos()] &= ~(US_LOWER_RIGHT|US_IS_UNIT);
    }
    numdeletedunit++;
    deleted_unitorstruct = true;
    un->remove();
    //if numdeletedunit > some_value then pack the unitpool
}

/// removes a structure from the map




void UnitAndStructurePool::removeStructure(Structure *st) {
    Uint16 curpos, x, y;

    //	unitandstructmat[st->getPos()] &= ~(US_LOWER_RIGHT|US_IS_STRUCTURE);
    structurepool[st->getStructnum()] = NULL;
    curpos = st->getPos();
    if (((StructureType*)st->getType())->isWall()) {
        updateWalls(st,false);
        unitandstructmat[curpos] &= ~(US_LOWER_RIGHT|US_IS_WALL);
    } else {
        for( y = 0; y<((StructureType *)st->getType())->getYsize(); y++ ) {
            for(x = 0; x<((StructureType *)st->getType())->getXsize(); x++) {
                if( ((StructureType *)st->getType())->isBlocked(y*((StructureType *)st->getType())->getXsize()+x) ) {
                    unitandstructmat[curpos+x] &= ~(US_LOWER_RIGHT|US_IS_STRUCTURE);
                }
            }
            curpos += ccmap->getWidth();
        }
        numdeletedstruct++; // don't count walls
    }
    deleted_unitorstruct = true;
    st->remove();
    //if numdeletedstruct > some_value then pack the structurepool
}

/** @brief scans neighbouring cells of a wall for walls and updates their
 * layer zero image
 * @param st pointer to the wall to scan around
 * @param add if true, the wall has been added, if false, the wall has been
 * removed */
void UnitAndStructurePool::updateWalls(Structure* st, bool add) {
    Structure* neighbour;
    StructureType* type;
    int cellpos;

    cellpos = st->getPos();
    type = ((StructureType*)st->getType());
    // left
    if(cellpos%ccmap->getWidth() > 0) {
        if( unitandstructmat[cellpos -1] & US_IS_WALL ) {
            // check if same type
            neighbour = structurepool[unitandstructmat[cellpos -1]&0xffff];
            if( neighbour->getType() == type ) {
                if (add) {
                    neighbour->changeImage(0, 2);
                    st->changeImage(0, 8);
                } else {
                    neighbour->changeImage(0,-2);
                }
            }
        }
    }
    // right
    if(cellpos%ccmap->getWidth() < ccmap->getWidth() - 1) {
        if( unitandstructmat[cellpos +1] & US_IS_WALL ) {
            // check if same type
            neighbour = structurepool[unitandstructmat[cellpos +1]&0xffff];
            if( neighbour->getType() == type ) {
                if (add) {
                    neighbour->changeImage(0, 8);
                    st->changeImage(0, 2);
                } else {
                    neighbour->changeImage(0,-8);
                }
            }
        }
    }
    // up
    if(cellpos/ccmap->getWidth() > 0) {
        if( unitandstructmat[cellpos -ccmap->getWidth()] & US_IS_WALL ) {
            // check if same type
            neighbour = structurepool[unitandstructmat[cellpos -ccmap->getWidth()]&0xffff];
            if( neighbour->getType() == type ) {
                if (add) {
                    neighbour->changeImage(0, 4);
                    st->changeImage(0, 1);
                } else {
                    neighbour->changeImage(0,-4);
                }
            }
        }
    }
    // down
    if(cellpos/ccmap->getWidth() < ccmap->getHeight() - 1) {
        if( unitandstructmat[cellpos +ccmap->getWidth()] & US_IS_WALL ) {
            // check if same type
            neighbour = structurepool[unitandstructmat[cellpos +ccmap->getWidth()]&0xffff];
            if( neighbour->getType() == type ) {
                if (add) {
                    neighbour->changeImage(0, 1);
                    st->changeImage(0, 4);
                } else {
                    neighbour->changeImage(0,-1);
                }
            }
        }
    }
}

void UnitAndStructurePool::showMoves() {
    Uint32 x;
    fprintf(stderr,"Current cells have US_MOVING_HERE set:\n"
            "cell\tvalue (US_MOVING_HERE == %u/%x)\n",US_MOVING_HERE,US_MOVING_HERE);
    for (x=0;x < (Uint32)ccmap->getWidth()*ccmap->getHeight();++x) {
        if (unitandstructmat[x]&US_MOVING_HERE)
            fprintf(stderr,"%i\t%u/%x\n",x,unitandstructmat[x],unitandstructmat[x]);
    }
}
