/*****************************************************************************
 * inifile.cpp - class to read strings/ints from INI formatted files
 *
 ****************************************************************************/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "inifile.h"
#include "vfs.h"

#include "memtrack.h"

#ifndef __GNUC__
int strcasecmp(char*, char*);
#endif

/** Function to extract a string from a ini file. The string
 * is malloced in this function so it should be freed.
 * @param the section in the file to extract string from.
 * @param the name of the string to extract.
 * @return the extracted string.
 */
char* INIFile::readString( char* section, char* value, char* deflt) {
    char* tmp;

    tmp = readString(section,value);
    if (tmp == NULL) {
        /* a new string is allocated because this guarentees
         * that the return value can be delete[]ed safely 
         */
        tmp = new char[strlen(deflt)+1];
        strncpy(tmp,deflt,strlen(deflt));
        tmp[strlen(deflt)] = 0x0;
    }
    return tmp;
}

char *INIFile::readString( char *section, char *value ) {
    char line[MAXLINELENGTH];
    char mline[MAXLINELENGTH];
    char compareval[MAXSTRINGLENGTH];
    char *returnval;
    char equals[2] = { '\0', '\0' }; /* Make sure it is zero to start off */
    mline[0] = '\0';
    compareval[0] = '\0'; /* Just precautions */
    if (inifile == NULL)
        return NULL;
    inifile->seekSet( 0 );
    /* Go to the right section */
    line[0] = '\0';
    while( strcasecmp(line, section) ) {
        if( inifile->getLine(line, 1024) == NULL ) {
            return NULL;
        }
        sscanf(line, "[%[^]]]", line );
    }

    /* Find the right value */
    while( strcasecmp(compareval, value) || equals[0] != '=' ) {
        if( inifile->getLine(line, 1024) == NULL ) {
            return NULL;
        }
        if(line[0] == ';')
            continue;
        if( sscanf(line, "[%[^]]]", mline ) == 1 ) {
            return NULL;
        }
        if(sscanf(line,"%[^= \t]%[=]%[^\n\r]", compareval, equals, mline ) != 3)
            equals[0] = '\0';
    }

    returnval = new char[strlen(mline)+1];
    strcpy(returnval, mline);
    return returnval;
}

/** Function to extract a integer value from a ini file. The value
 * can be given in hex if it starts with 0x.
 * @param the section in the file to extract values from.
 * @param the name of the value to extract.
 * @return the value.
 */

int INIFile::readInt( char* section, char* value, Uint32 deflt ) {
    Uint32 tmp;

    tmp = readInt(section,value);
    if (tmp == INIERROR)
        return deflt;
    else
        return tmp;
}

int INIFile::readInt( char* section, char* value ) {
    char compareval[MAXSTRINGLENGTH];
    char line[MAXLINELENGTH];
    char mline[MAXLINELENGTH];
    int returnval;
    char equals[2] = { '\0', '\0' };

    inifile->seekSet( 0 );

    compareval[0] = '\0';
    mline[0] = '\0';
    /* Go to the right section */
    line[0] = '\0';
    while( strcasecmp(line, section) ) {
        if( inifile->getLine(line, 1024) == NULL ) {
            return INIERROR; /* Largest possible int is errorcode */
        }
        sscanf(line, "[%[^]]]", line );
    }

    /* Find the right value */
    while( strcasecmp(compareval, value) || equals[0] != '=' ) {
        if( inifile->getLine(line, 1024) == NULL ) {
            return INIERROR;
        }
        if(line[0] == ';')
            continue;
        if( sscanf(line, "[%[^]]]", mline ) == 1 ) {
            return INIERROR;
        }
        if(sscanf(line,"%[^= \t]%[=]%i",compareval, equals, &returnval ) != 3)
            equals[0] = '\0';
    }

    return returnval;
}

/** Get the next line in the current section.
 * @returns NULL if the end of the section is encountered or an error
 * occurs, a pointer to a string on succes.
 */
char *INIFile::getLine() {
    char line[MAXLINELENGTH];
    char *returnval;

    line[0] = ';';
    while( line[0] == ';' || line[0] == '\n' || line[0] == '\r' ) {
        if( inifile->getLine(line, 1024) == NULL ) {
            return NULL;
        }

        if( sscanf(line, "[%[^]]]", line ) == 1 )
            return NULL;

    }

    returnval = new char[strlen(line)+1];
    strcpy(returnval, line);
    return returnval;
}

/* Sets the file ptr to the beginin of a section.
 * @param the name of the section to seek to.
 * @returns 0 if section is not found, 1 if found
 */
int INIFile::seekSection( char *section ) {
    char line[MAXLINELENGTH];
    /* Go to the right section */
    inifile->seekSet( 0 );
    line[0] = '\0';
    while( strcasecmp(line, section) ) {
        if( inifile->getLine(line, 1024) == NULL ) {
            return 0; /* Didn't find the section */
        }
        sscanf(line, "[%[^]]]", line );
    }
    return 1;
}


/** Constructor, opens the file
 * @param the name of the inifile to open.
 */
INIFile::INIFile(char *filename) {
    inifile = VFS_Open(filename);

    if(inifile == NULL)
        throw new INInotFound(filename);
}

/** Destructor, closes the file */
INIFile::~INIFile() {
    VFS_Close(inifile);
}
